#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "rate.hh"
#include "rate/backward_euler.hh"
#include "rate/newmark.hh"

template <class Vector, class Matrix, class Function, int dimension>
std::shared_ptr<RateUpdater<Vector, Matrix, Function, dimension>>
initRateUpdater(Config::scheme scheme,
                const std::vector<Function>& velocityDirichletFunctions,
                const std::vector<Dune::BitSetVector<dimension>>& velocityDirichletNodes,
                const Matrices<Matrix>& matrices,
                const std::vector<Vector>& u_initial,
                const std::vector<Vector>& v_initial,
                const std::vector<Vector>& a_initial) {
  switch (scheme) {
    case Config::Newmark:
      return std::make_shared<Newmark<Vector, Matrix, Function, dimension>>(
          matrices, u_initial, v_initial, a_initial, velocityDirichletNodes,
          velocityDirichletFunctions);
    case Config::BackwardEuler:
      return std::make_shared<
          BackwardEuler<Vector, Matrix, Function, dimension>>(
          matrices, u_initial, v_initial, a_initial, velocityDirichletNodes,
          velocityDirichletFunctions);
    default:
      assert(false);
  }
}

#include "rate_tmpl.cc"
