#include <cmath>

#include "sliplawstateupdater.hh"
#include "../../tobool.hh"

template <class ScalarVector, class Vector>
SlipLawStateUpdater<ScalarVector, Vector>::SlipLawStateUpdater(
    const std::vector<ScalarVector>& _alpha_initial,
    const std::vector<Dune::BitSetVector<1>>& _nodes, const std::vector<double>& _L,
    const std::vector<double>& _V0)
    : alpha(_alpha_initial), nodes(_nodes), L(_L), V0(_V0) {}

template <class ScalarVector, class Vector>
void SlipLawStateUpdater<ScalarVector, Vector>::nextTimeStep() {
  alpha_o = alpha;
}

template <class ScalarVector, class Vector>
void SlipLawStateUpdater<ScalarVector, Vector>::setup(double _tau) {
  tau = _tau;
}

template <class ScalarVector, class Vector>
void SlipLawStateUpdater<ScalarVector, Vector>::solve(
  const std::vector<Vector>& velocity_field) {

  for (size_t i = 0; i < alpha.size(); ++i) {
      const auto& velocity_fieldi = velocity_field[i];
      const auto& nodesi = nodes[i];
      auto& alphai = alpha[i];
      auto& alpha_oi = alpha_o[i];

      for (size_t j = 0; j < nodesi.size(); ++j) {
        if (not toBool(nodesi[j]))
          continue;

        double const V = velocity_fieldi[j].two_norm();
        double const mtoL = -tau * V / L[i];
        alphai[j] = (V <= 0) ? alpha_oi[j] : std::expm1(mtVoL) * std::log(V / V0[i]) +
                                               alpha_oi[j] * std::exp(mtVoL);
      }
   }
}

template <class ScalarVector, class Vector>
void SlipLawStateUpdater<ScalarVector, Vector>::extractAlpha(
    std::vector<ScalarVector>& _alpha) {
  _alpha = alpha;
}

template <class ScalarVector, class Vector>
std::shared_ptr<StateUpdater<ScalarVector, Vector>>
SlipLawStateUpdater<ScalarVector, Vector>::clone() const {
  return std::make_shared<SlipLawStateUpdater<ScalarVector, Vector>>(*this);
}
